@@ -25,494 +25,16 @@ import main
 import battle
 import monster
 
+debug = 0
+if debug == 1:
+	from time import clock
+
 #set to 1 whenever the player really shouldn't be moving.
 global has_dialog
 has_dialog = 0
 
 
 
-#Activate one line of scripting. See scripting.txt for details.
-#returns either 0<= (failure), >0 (success),
-#or "end" (end script).
-#x and y are absolute values.
-
-#Removed tile_type, I think it's unused now.
-# def activate_line(x, y, z, command):
-#
-# 	global has_dialog
-# 	return_num = 0
-# 	switch = command.split(None, 1)[0].lower()
-# 	global just_found
-#
-# 	if command.isdigit() == 1: #if command is a number
-# 		return_num = int(command)
-# 	elif command[:1] == "-" and command[1:].isdigit() == 1:
-# 		return_num = int(command)
-#
-# 	elif switch == "addpix": #add a picture to the tile.
-# 		g.maps[z].field[y][x].add_pix(g.tiles[command.split(None, 1)[1]])
-# 		return_num = 1
-#
-# 	elif switch == "addskill": #add a skill
-# 		temp = g.findskill(command.split(None, 1)[1])
-# 		if temp == -1:
-# 			print "Unknown skill: " + command.split(None, 2)[1]
-# 			return_num = 0
-# 		else:
-# 			return_num = g.add_skill(temp)
-# 	elif switch == "attack":  #cause creature to attack
-# 		temp = -1
-# 		if command.split(None, 2)[1].lower() == "random":
-# 			#battle random monster
-# 			i = g.mapname2zgrid(command.split(None, 2)[2])
-# 			temp = monster.find_level_monster(i)
-# 		else:
-# 			#battle named monster
-# 			for monster_num in range(len(monster.monster_groups)):
-# 				if monster.monster_groups[monster_num].name.lower() == \
-# 				command.split(None, 1)[1].lower():
-# 					temp = monster_num
-# 					break
-# 			else:
-# 				print "monster " + command.split()[1] + " not found"
-# 		#if there exists a monster to battle:
-# 		if temp != -1:
-# 			return_num = main.start_battle(temp)
-#
-# 	elif switch == "damage_monster":  #In battle, hurt monster for either
-# 			#adj_attack points, or Command points. *NOT* reduced by armor.
-# 		if g.cur_window != "battle":
-# 			print "damage_monster called outside of battle."
-# 			return -1
-# 		temp_command_array = command.split(None, 1)
-#
-# 		mon_num = battle.select_monster()
-# 		if mon_num == -1: return 0
-# 		if len(temp_command_array) == 1: #Basic damage_monster command.
-# 			battle.monster_hurt(mon_num, player.adj_attack)
-# 			return_num = 1
-# 		else: #Of the format damage_monster Command.
-# 			set_to = activate_line(x, y, z, temp_command_array[1])
-# 			battle.monster_hurt(mon_num, set_to)
-# 			return_num = 1
-#
-# 	elif switch == "delpix": #remove a picture from the tile.
-# 		try:
-# 			g.maps[z].field[y][x].del_pix(
-# 				g.tiles[command.split(None, 1)[1]])
-# 			main.refreshmap()
-# 			return_num = 1
-# 		except ValueError:
-# 			return_num = 0
-#
-# 	elif switch == "dialog":  #show a message in a dialog box.
-# 		main.show_dialog(command.split(None, 1)[1])
-# 		return_num = 1
-#
-# 	elif switch == "die":  #process endgame.txt script
-# 		player.hp = -1
-# 		for line in g.endgame_act:
-# 			activate_line(x, y, z, line)
-# 		return_num = "end"
-#
-# 	elif switch == "end":  #end the script
-# 		return_num = "end"
-#
-# 	elif switch  == "find":
-# 		find_info = command.split("\"", 2)
-# 		founditem = find_info[1].strip()
-# 		amount = find_info[2].strip()
-# 		main.print_message("You found " + amount + " " + founditem + ".")
-# 		has_dialog = 1
-# 		if main.show_yesno("You found " + amount + " " + founditem +
-# 				"! Would you like to pick it up ?"):
-# 			if(founditem == "gold"):
-# 				player.give_stat("gold", int(amount))
-# 				main.print_message("You picked up " + amount + " " +
-# 						founditem + ".")
-# 				has_dialog = 0
-# 				return 1
-# 			if( g.item.take_inv_item(g.item.finditem(founditem)) != -1):
-# 				main.print_message("You picked up " + amount + " " +
-# 						founditem + ".")
-# 				has_dialog = 0
-# 				return 1
-# 			else:
-# 				main.print_message("You have no room for the " +
-# 						founditem + ".")
-# 				has_dialog = 0
-# 				return 0
-# 		else:
-# 			has_dialog = 0
-# 			return 0
-#
-# 	elif switch == "give":  #Change stats. Note that negatve numbers also work
-# 		switch2 = command.split(None, 2)[1].lower()
-# 		#interpret the second part of the command
-# 		set_to = command.split(None, 2)[2]
-# 		if switch2 != "name":
-# 			set_to = activate_line(x, y, z, command.split(None, 2)[2])
-#
-# 		if switch2 == "name":
-# 			#note that name is not interpreted as a command. See above.
-# 			player.name = interpret_line(set_to)
-# 		elif switch2 == "hp": player.give_stat("hp", int(set_to))
-# 		elif switch2 == "ep": player.give_stat("ep", int(set_to))
-# 		elif switch2 == "maxhp": player.give_stat("maxhp", int(set_to))
-# 		elif switch2 == "maxep": player.give_stat("maxep", int(set_to))
-# 		elif switch2 == "attack": player.give_stat("attack", int(set_to))
-# 		elif switch2 == "defense": player.give_stat("defense", int(set_to))
-# 		elif switch2 == "adj_maxhp": player.give_stat("adj_maxhp", int(set_to))
-# 		elif switch2 == "adj_maxep": player.give_stat("adj_maxep", int(set_to))
-# 		elif switch2 == "adj_attack": player.give_stat("adj_attack", int(set_to))
-# 		elif switch2 == "adj_defense": player.give_stat("adj_defense", int(set_to))
-# 		elif switch2 == "gold": player.give_stat("gold", int(set_to))
-# 		elif switch2 == "exp": player.add_exp(int(set_to))
-# 		elif switch2 == "skillpoints":
-# 			player.give_stat("skillpoints", int(set_to))
-# 		else:
-# 			print "Unknown stat: " + switch2
-# 		main.recalc_stats()
-# 		if set_to == 0: return_num = 0
-# 		else: return_num = 1
-#
-# 	elif switch == "hero":  #change hero pix
-# 		player.cur_hero = "people/" + command.split(None, 2)[1] + ".png"
-# 		main.refreshmap()
-#
-# 	elif switch == "hurt":  #hurt player (reduced by armor)
-# 		#interpret the second part of the command
-# 		set_to = command.split(None, 1)[1]
-# 		set_to = activate_line(x, y, z, command.split(None, 1)[1])
-# 		if set_to == "end": return "end"
-# 		damage = g.die_roll(1, int(set_to) + 2)
-# 		damage = damage - g.die_roll(1, player.adj_defense + 2)
-# 		player.give_stat("hp", -1*damage)
-# 		return_num = 1
-#
-# 	elif switch == "hurt_monster":  #In battle, hurt monster for either
-# 			#adj_attack points, or Command points. Reduced by armor.
-# 		if g.cur_window != "battle":
-# 			print "hurt_monster called outside of battle."
-# 			return -1
-# 		temp_command_array = command.split(None, 1)
-#
-# 		mon_num = battle.select_monster()
-# 		if mon_num == -1: return 0
-# 		if len(temp_command_array) == 1: #Basic hurt_monster command.
-# 			battle.attack_monster(mon_num, player.adj_attack)
-# 			return_num = 1
-# 		else: #Of the format hurt_monster Command.
-# 			set_to = activate_line(x, y, z, temp_command_array[1])
-# 			battle.attack_monster(mon_num, set_to)
-# 			return_num = 1
-#
-# 	elif switch == "if":
-# 		temp = command.split(None, 1)[1].strip()
-# 		compare = 0
-# 		if (temp.find("<=") > 0): #note that <= and >= need to come before =
-# 			line_return1 = \
-# 				activate_line(x, y, z, temp.split("<=", 1)[0].strip())
-# 			line_return2 = \
-# 				activate_line(x, y, z, temp.split("<=", 1)[1].strip())
-# 			if (line_return1 <= line_return2):
-# 				compare = 1
-# 		elif (temp.find(">=") > 0):
-# 			line_return1 = \
-# 				activate_line(x, y, z, temp.split(">=", 1)[0].strip())
-# 			line_return2 = \
-# 				activate_line(x, y, z, temp.split(">=", 1)[1].strip())
-# 			if (line_return1 >= line_return2):
-# 				compare = 1
-# 		elif (temp.find("=") > 0): #this catches = and ==
-# 			line_return1 =activate_line(x, y, z, temp.split("=", 1)[0].strip())
-# 			line_return2 =activate_line(x, y, z, temp.split("=", 1)[1].strip())
-# 			if (line_return1 == line_return2):
-# 				compare = 1
-# 		elif (temp.find("<") > 0):
-# 			line_return1 =activate_line(x, y, z, temp.split("<", 1)[0].strip())
-# 			line_return2 =activate_line(x, y, z, temp.split("<", 1)[1].strip())
-# 			if (line_return1 < line_return2):
-# 				compare = 1
-# 		elif (temp.find(">") > 0):
-# 			line_return1 =activate_line(x, y, z, temp.split(">", 1)[0].strip())
-# 			line_return2 =activate_line(x, y, z, temp.split(">", 1)[1].strip())
-# 			if (line_return1 > line_return2):
-# 				compare = 1
-# 		else: #of the form eg "if var something
-# 			line_return1 = activate_line(x, y, z, temp)
-# 			line_return2 = 0
-# 			if (line_return1 > 0):
-# 				compare = 1
-#
-# 		if (line_return1 == "end"):  #something that should end the script
-# 			return_num = "end"
-# 		elif (line_return2 == "end"):
-# 			return_num = "end"
-# 		elif (compare == 1): #first
-# 			return "if"
-# 		else:  #second
-# 			return "else"
-#
-# 	elif switch == "info":  #display line of text in textbox below main screen
-# 		has_dialog = 1
-# 		main.print_message(command.split(None, 1)[1])
-# 		has_dialog = 0
-# 		return_num = 1
-#
-# 	elif switch == "is_equipped": #is the given item equipped?
-# 		match = command.split(None, 1)[1].strip().lower()
-# 		for item_title in player.equip:
-# 			if item_title == -1: continue
-# 			if g.item.item[item_title].name.lower() == match:
-# 				return 1
-# 		return 0
-#
-# 	elif switch == "item":  #give item
-# 		if g.item.take_inv_item(g.item.finditem(command.split(None, 1)[1])) == -1:
-# 			return_num = 0
-# 		else: return_num = 1
-#
-# 	elif switch == "lose":
-# 		player.hp = -1
-# 		main.close_window()
-# 		return_num = "end"
-#
-# 	elif switch == "mapstat":
-# 		switch2 = command.split(None, 2)[1].lower()
-#
-# 		if switch2 == "addmonster":
-# 			g.maps[z].monster.append(command.split(None, 2)[2].strip())
-# 			return_num = 1
-# 		elif switch2 == "delmonster":
-# 			try:
-# 				g.maps[z].monster.remove(command.split(None, 2)[2].strip())
-# 				return_num = 1
-# 			except ValueError: return_num = 0
-# 		elif switch2 == "hero_bg":
-# 			try: g.maps[z].hero_suffix = command.split(None, 2)[2].strip()
-# 			except IndexError: g.maps[z].hero_suffix = ""
-# 		elif switch2 == "battle_bg":
-# 			g.maps[z].battle_background = \
-# 							g.backgrounds[command.split(None, 2)[2].strip()]
-# 			g.maps[z].battle_background_name = \
-# 							command.split(None, 2)[2].strip()
-# 	elif switch == "monster_stat": #In battle, return the stats of the
-# 									#given monster
-# 		if g.cur_window != "battle":
-# 			print "monster_stat called outside of battle."
-# 			return -1
-# 		switch2 = command.split(None, 2)[1].lower().strip()
-#
-# 		#Choose the monster to examine
-# 		if switch2 == "select":
-# 			mon_num = battle.select_monster()
-# 			if mon_num == -1: return -1
-# 		elif switch2.isdigit() or \
-# 						(switch2[:1] == "-" and switch2[1:].isdigit()):
-# 			mon_num = int(switch2)
-# 			if mon_num == -1: return -1
-# 		else:
-# 			print "monster_stat error in command: " + command
-# 			return -1
-#
-# 		#Choose the stat to examine
-# 		switch3 = command.split(None, 2)[2].lower().strip()
-# 		if switch3 == "name":
-# 			return battle.monster_list[mon_num].name
-# 		elif switch3 == "hp":
-# 			return battle.monster_list[mon_num].hp
-# 		elif switch3 == "maxhp":
-# 			return battle.monster_list[mon_num].maxhp
-# 		elif switch3 == "attack":
-# 			return battle.monster_list[mon_num].attack
-# 		elif switch3 == "defense":
-# 			return battle.monster_list[mon_num].defense
-# 		else:
-# 			print "Bad stat of " + switch3 + " used in monster_stat."
-# 			return -1
-#
-# 	elif switch == "monster_select":  #In battle, ask the player to
-# 										#select a monster
-# 		if g.cur_window != "battle":
-# 			print "monster_select called outside of battle."
-# 			return -1
-#
-# 		mon_num = battle.select_monster()
-# 		if mon_num == -1: return -1
-# 		return mon_num
-#
-# 	elif switch == "move":  #move the player
-# 		curr_zgrid = g.zgrid
-# 		temp_zgrid = g.mapname2zgrid(command.split()[1])
-# 		if temp_zgrid != -1:
-# 			g.zgrid = temp_zgrid
-# 		g.xgrid = int(command.split()[2])
-# 		g.ygrid = int(command.split()[3])
-# 		if curr_zgrid != g.zgrid:
-# 			main.process_onload()
-# 		main.refreshmap()
-# 		g.allow_move = 0
-# 		return_num = 1
-#
-# 	elif switch == "pass":  #do nothing
-# 		return_num = 1
-#
-# 	elif switch == "pix":  #change tile picture
-# 		g.maps[z].field[y][x].pix = \
-# 			g.tiles[command.split(None, 2)[1]]
-# 		g.maps[z].field[y][x].addpix = []
-# 		return_num = 1
-#
-# 	elif switch == "printvars": #debug: print all variables
-# 		print g.var_list
-#
-# 	elif switch == "question":  #yes/no dialog box
-# 		has_dialog = 1
-# 		if main.show_yesno(command.split(None, 1)[1]):
-# 			return_num = 1
-# 		else:
-# 			return_num = 0
-# 		has_dialog = 0
-#
-# 	elif switch == "refresh": #refresh the screen manually.
-# 		main.refreshmap()
-# 		return_num = 1
-#
-# 	elif switch == "rng":  #Random Number Generator
-# 		temp = g.die_roll(1, int(command.split(None, 2)[2]))
-# 		if int(command.split(None, 2)[1]) >= temp:
-# 			return_num = temp
-# 		else: return_num = 0
-#
-# 	elif switch == "run":  #Run the actions of a different tile.
-# 		temp_zgrid = g.mapname2zgrid(command.split()[1])
-# 		if temp_zgrid == -1:
-# 			temp_zgrid = g.zgrid
-# 		x = int(command.split()[2])
-# 		y = int(command.split()[3])
-# 		return_num = 1
-# 		#if there are no actions, leave immediately
-# 		if len(g.maps[temp_zgrid].field[y][x].actions) == 0: return 1
-# 		i = 0
-# 		#go through all action lines.
-# 		return activate_lines(x, y, z, g.maps[temp_zgrid].field[y][x].actions)
-#
-# 	elif switch == "set":  #set variable (in g.var_list)
-# 		command2 = command.split(None, 1)[1].strip()
-# 		if -1 != command2.find("="):
-# 			message = interpret_line(command2.split("=", 1)[1].strip())
-# 			g.var_list[command2.split("=", 1)[0].lower().strip()] = message
-# 			return_num = 1
-# 		elif -1 != command2.find("+"):
-# 			if 0 == \
-# 				g.var_list.has_key(command2.split("+", 1)[0].lower().strip()):
-# 				g.var_list[command2.split("+", 1)[0].lower().strip()] = 0
-# 			message = interpret_line(command2.split("+", 1)[1].strip())
-# 			#The reason I don't just use a += command is that g.var_list[x]
-# 			#may be a string.
-# 			g.var_list[command2.split("+", 1)[0].lower().strip()] = \
-# 			int(g.var_list[command2.split("+", 1)[0].lower().strip()]) + \
-# 			int(message)
-# 			return_num = 1
-# 		elif -1 != command2.find("-"):
-# 			if 0 == \
-# 				g.var_list.has_key(command2.split("-", 1)[0].lower().strip()):
-# 				g.var_list[command2.split("-", 1)[0].lower().strip()] = 0
-# 			message = interpret_line(command2.split("-", 1)[1].strip())
-# 			g.var_list[command2.split("-", 1)[0].lower().strip()] = \
-# 			int(g.var_list[command2.split("-", 1)[0].lower().strip()]) - \
-# 			int(message)
-# 			return_num = 1
-# 		else: #no modifier; use =
-# 			message = interpret_line(command.split(None, 2)[2].strip())
-# 			g.var_list[command.split(None, 2)[1].lower()] = message
-# 			return_num = 1
-#
-# 	elif switch == "stat":  #return stat
-# 		switch2 = command.split(None, 2)[1].lower()
-# 		if switch2 == "name":
-# 			return_num = player.name
-# 		elif switch2 == "hp":
-# 			return_num = player.hp
-# 		elif switch2 == "ep":
-# 			return_num = player.ep
-# 		elif switch2 == "maxhp":
-# 			return_num = player.adj_maxhp
-# 		elif switch2 == "maxep":
-# 			return_num = player.adj_maxep
-# 		elif switch2 == "attack":
-# 			return_num = player.adj_attack
-# 		elif switch2 == "defense":
-# 			return_num = player.adj_defense
-# 		elif switch2 == "gold":
-# 			return_num = player.gold
-# 		elif switch2 == "exp":
-# 			return_num = player.exp
-# 		elif switch2 == "level":
-# 			return_num = player.level
-# 		elif switch2 == "skillpoints":
-# 			return_num = player.skillpoints
-# 		else:
-# 			print "Unknown stat: " + switch2
-# 			return_num = 0
-#
-# 	elif switch == "store":  #enter store
-# 		has_dialog = 1
-# 		main.enter_store(command.split(None, 1)[1])
-# 		has_dialog = 0
-# 		return_num = 1
-#
-# 	elif switch == "take":  #Drop item, by name.
-# 		temp = g.item.find_inv_item(g.item.finditem(command.split(None, 1)[1]))
-# 		if temp == -1:
-# 			return_num = 0
-# 		else:
-# 			g.item.drop_inv_item(temp)
-# 			return_num = 1
-#
-# 	elif switch == "var":  #return variable (in g.var_list)
-# 		command2 = command.split(None, 1)[1].lower().strip()
-# 		if (g.var_list.has_key(command2)):
-# 			try:
-# 				if g.var_list[command2].isdigit():
-# 					return_num = int(g.var_list[command2])
-# 				else: return_num = g.var_list[command2]
-# 			except AttributeError:
-# 				return_num = int(g.var_list[command2])
-# 		else: return_num = 0
-#
-# 	elif switch == "walk":  #change tile walkability
-# 		g.maps[z].field[y][x].walk = \
-# 				int(command.split(None, 1)[1])
-# 		return_num = 1
-#
-# 	elif switch == "wall_n":  #change tile wall values
-# 		g.maps[z].field[y][x].wall_n = \
-# 				int(command.split(None, 1)[1])
-# 		return_num = 1
-# 	elif switch == "wall_s":  #change tile wall values
-# 		g.maps[z].field[y][x].wall_s = \
-# 				int(command.split(None, 1)[1])
-# 		return_num = 1
-# 	elif switch == "wall_e":  #change tile wall values
-# 		g.maps[z].field[y][x].wall_e = \
-# 				int(command.split(None, 1)[1])
-# 		return_num = 1
-# 	elif switch == "wall_w":  #change tile wall values
-# 		g.maps[z].field[y][x].wall_w = \
-# 				int(command.split(None, 1)[1])
-# 		return_num = 1
-#
-# 	elif switch == "win":  #process wingame.txt script
-# #		for line in g.wingame_act:
-# 		activate_lines(g.xgrid, g.ygrid, g.zgrid, g.wingame_act)
-# 		return_num = 1
-#
-# 	else:
-# 		print "Bad action of: " + switch + " given."
-# 		return_num = "bad"
-# 	return return_num
 
 
 #given a string such as info or question uses, interpret all ~Action~
@@ -796,12 +318,18 @@ def run_command(x, y, z, command):
 	elif switch == "end":  #end the script
 		if check_num_args(len(arg_list), 0, 0, "end"):
 			return script_end(x, y, z, arg_list)
+	elif switch == "equip":  #adjust equipment
+		if check_num_args(len(arg_list), 2, 2, "equip"):
+			return script_equip(x, y, z, arg_list)
 	elif switch  == "find":
 		if check_num_args(len(arg_list), 2, 2, "find"):
 			return script_find(x, y, z, arg_list)
 	elif switch == "gamestat":
 		if check_num_args(len(arg_list), 1, 1, "gamestat"):
 			return script_gamestat(x, y, z, arg_list)
+	elif switch == "generic_dialog":  #Create a generic dialog box
+		if check_num_args(len(arg_list), 2, 999, "generic_dialog"):
+			return script_generic_dialog(x, y, z, arg_list)
 	elif switch == "give":  #Change stats. Note that negative numbers also work
 		if check_num_args(len(arg_list), 2, 2, "give"):
 			return script_give(x, y, z, arg_list)
@@ -835,6 +363,9 @@ def run_command(x, y, z, command):
 	elif switch == "lose":
 		if check_num_args(len(arg_list), 0, 0, "lose"):
 			return script_lose(x, y, z, arg_list)
+	elif switch == "mapspot":
+		if check_num_args(len(arg_list), 4, 4, "mapspot"):
+			return script_mapspot(x, y, z, arg_list)
 	elif switch == "mapstat":
 		if check_num_args(len(arg_list), 2, 2, "mapstat"):
 			return script_mapstat(x, y, z, arg_list)
@@ -1028,6 +559,50 @@ def script_die(x, y, z, argument_array):  #process endgame.txt script
 def script_end(x, y, z, argument_array):  #end the script
 	return "end"
 
+def script_equip(x, y, z, argument_array): #adjust equipment
+	if check_types_args(argument_array, [1, 1], "equip") == 0:
+		return "bad"
+
+	switch2 = argument_array[0][0][1:-1].lower()
+	switch3 = argument_array[1][0][1:-1].lower()
+
+	if switch2 == "in_slot":
+		if switch3 == "weapon": equip_loc = 0
+		elif switch3 == "armor": equip_loc = 1
+		elif switch3 == "shield": equip_loc = 2
+		elif switch3 == "helmet": equip_loc = 3
+		elif switch3 == "gloves": equip_loc = 4
+		elif switch3 == "boots": equip_loc = 5
+		else:
+			print "unknown equipment space of " + switch3
+			return "bad"
+		if player.equip[equip_loc] == -1: return "\"\""
+		return "\""+g.item.item[player.equip[equip_loc]].name+"\""
+	if switch2 == "has":
+		for i in range(len(player.equip)):
+			if switch3 == g.item.item[player.equip[i]].name.lower():
+				return 1
+		return 0
+	elif switch2 == "take":
+		for i in range(len(player.equip)):
+			if switch3 == g.item.item[player.equip[i]].name.lower():
+				player.equip[i] = -1
+				return 1
+		return 0
+	elif switch2 == "give":
+		item_loc = g.item.finditem(switch3)
+		if item_loc == -1:
+			print "unknown item " + switch3
+			return "bad"
+		if g.item.item[item_loc].type > 5:
+			print "item "+switch3+" cannot be worn"
+			return "bad"
+		player.equip[g.item.item[item_loc].type] = item_loc
+		return 1
+	else:
+		print "unknown switch for equip(): "+switch2
+		return "bad"
+
 def script_find(x, y, z, argument_array):
 	if check_types_args(argument_array, [1, 2], "find") == 0:
 		return "bad"
@@ -1078,11 +653,27 @@ def script_gamestat(x, y, z, argument_array):
 		return int(g.xgrid)
 	elif switch2 == "y":
 		return int(g.ygrid)
+	elif switch2 == "newx":
+		return int(x)
+	elif switch2 == "newy":
+		return int(y)
 	elif switch2 == "mapname":
 		return "\"" + g.maps[g.zgrid].name + "\""
 	else: print "Unknown stat: " + switch2
 	return "bad"
 
+def script_generic_dialog(x, y, z, argument_array):  #Custom dialog box.
+	if check_types_args(argument_array, [1], "generic_dialog") == 0:
+		return "bad"
+	line = argument_array.pop(0)
+	line = insert_newlines(line[0][1:-1])
+	dialog_array = []
+	for entry in argument_array:
+		if check_types_args([entry], [1], "generic_dialog") == 0:
+			return "bad"
+		dialog_array.append(entry[0][1:-1])
+	return main.show_popup(line, dialog_array)
+
 def script_give(x, y, z, argument_array):  #Change stats. Note that negative numbers also work
 	if check_types_args(argument_array, [1, 2], "give") == 0:
 		return "bad"
@@ -1201,11 +792,13 @@ def script_if(x, y, z, argument_array):
 		return 0
 
 def script_info(x, y, z, argument_array):  #display line of text in textbox below main screen
-	if check_types_args(argument_array, [1], "info") == 0:
+	if check_types_args(argument_array, [2], "info") == 0:
 		return "bad"
 
 	has_dialog = 1
-	main.print_message(argument_array[0][0][1:-1])
+	if argument_array[0][1] == 0: message_txt = str(argument_array[0][0])
+	else: message_txt = argument_array[0][0][1:-1]
+	main.print_message(message_txt)
 	has_dialog = 0
 	return 1
 
@@ -1218,19 +811,57 @@ def script_inv(x, y, z, argument_array): #inventory functions.
 		print "Item " + argument_array[1][0][1:-1] + " does not exist."
 		return "bad"
 
-	if argument_array[0][0][1:-1].lower() == "has":
+	switch2 = argument_array[0][0][1:-1].lower()
+
+	if switch2 == "has":
 		if g.item.find_inv_item(temp) > -1: return 1
 		else: return 0
-	elif argument_array[0][0][1:-1].lower() == "take":
+	elif switch2 == "take":
 		temp = g.item.find_inv_item(temp)
 		if temp > -1:
 			g.item.drop_inv_item(temp)
 			return 1
 		else: return 0
-	elif argument_array[0][0][1:-1].lower() == "give":
+	elif switch2 == "give":
 		if g.item.take_inv_item(temp) == -1:
 			return 0
 		else: return 1
+	elif switch2 == "use":
+		tmptype = g.item.item[temp].type
+		if (tmptype != 11 and tmptype != 12 and tmptype != 14 and tmptype != 15
+							and tmptype != 16 and tmptype != 17):
+			print "item " + g.item.item[temp].name + " cannot be used."
+			return "bad"
+		if g.cur_window == "battle" and (tmptype != 16):
+			battle.useitem(temp, 1)
+		elif g.cur_window != "battle" and (tmptype == 11 or tmptype == 16 or
+							tmptype == 17):
+			main.inv.use_item(temp)
+			return 1
+		else:
+			print "item " + g.item.item[temp].name + " cannot be used in " \
+				+ g.cur_window
+			return "bad"
+	elif switch2 == "type":
+		return g.item.item[temp].type
+	elif switch2 == "quality":
+		return g.item.item[temp].quality
+	elif switch2 == "price":
+		return g.item.item[temp].price
+	elif switch2 == "value":
+		return g.item.item[temp].value
+	elif switch2 == "description":
+		return "\""+g.item.item[temp].description+"\""
+	elif switch2 == "picturename":
+		return "\""+g.item.item[temp].picturename+"\""
+	elif switch2 == "hp_bonus":
+		return g.item.item[temp].hp_bonus
+	elif switch2 == "ep_bonus":
+		return g.item.item[temp].ep_bonus
+	elif switch2 == "attack_bonus":
+		return g.item.item[temp].attack_bonus
+	elif switch2 == "defense_bonus":
+		return g.item.item[temp].defense_bonus
 	else:
 		print "Unknown switch " + argument_array[0][0][1:-1] + "found."
 		return "bad"
@@ -1272,6 +903,52 @@ def script_lose(x, y, z, argument_array):
 	main.close_window()
 	return "end"
 
+def script_mapspot(x, y, z, argument_array):
+	if check_types_args(argument_array, [1, 0, 0, 1], "mapspot") == 0:
+		return "bad"
+
+	tmpzgrid = g.mapname2zgrid(argument_array[0][0][1:-1])
+	if tmpzgrid == -1: return "bad"
+
+	tmpxgrid = argument_array[1][0]
+	tmpygrid = argument_array[2][0]
+
+	switch2 = argument_array[3][0][1:-1].lower()
+
+	if switch2 == "walk":
+		return g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].walk
+	if switch2 == "pix":
+		return "\""+g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].name+"\""
+	if switch2 == "num_of_dropped":
+		return len(g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].items)
+	if switch2 == "num_of_addpix":
+		return len(g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].addpix)
+	if switch2 == "num_of_addoverpix":
+		return len(g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].addoverpix)
+	if switch2 == "wall_n":
+		return g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].wall_n
+	if switch2 == "wall_s":
+		return g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].wall_s
+	if switch2 == "wall_w":
+		return g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].wall_w
+	if switch2 == "wall_e":
+		return g.maps[tmpzgrid].field[tmpygrid][tmpxgrid].wall_e
+	if switch2 == "within_bounds":
+		if tmpygrid < len(g.maps[tmpzgrid].field):
+			if tmpxgrid < len(g.maps[tmpzgrid].field[tmpygrid]): return 1
+			else: return 0
+		else: return 0
+	if switch2 == "y_bound":
+		return len(g.maps[tmpzgrid].field)
+	if switch2 == "x_bound":
+		return len(g.maps[tmpzgrid].field[0])
+	else:
+		print "unknown switch "+switch2+" in mapspot."
+		print "possible: walk, pix, num_of_dropped, num_of_addpix,"
+		print "num_of_addoverpix, wall_n, wall_s, wall_w, wall_e,"
+		print "within_bounds, y_bound, or x_bound."
+		return "bad"
+
 def script_mapstat(x, y, z, argument_array):
 	if check_types_args(argument_array, [1, 1], "mapstat") == 0:
 		return "bad"
@@ -1300,7 +977,7 @@ def script_mapstat(x, y, z, argument_array):
 	elif switch2 == "change_titlebar":
 		switch3 = interpret_line(switch3)
 		g.game_name = switch3
-		main.window_main.title(g.game_name)
+		g.window_main.title(g.game_name)
 		return 1
 
 #In battle, change the stats of the given monster
@@ -1395,6 +1072,9 @@ def script_move(x, y, z, argument_array):  #move the player
 	if check_types_args(argument_array, [1, 0, 0], "move") == 0:
 		return "bad"
 
+	if debug == 1:
+		tmp = clock()
+
 	g.xgrid = int(argument_array[1][0])
 	g.ygrid = int(argument_array[2][0])
 
@@ -1407,6 +1087,8 @@ def script_move(x, y, z, argument_array):  #move the player
 		main.process_onload()
 	main.refreshmap()
 	g.allow_move = 0
+	if debug == 1:
+		print clock() - tmp
 	return 1
 
 def script_pass(x, y, z, argument_array):  #do nothing
@@ -1417,6 +1099,7 @@ def script_pix(x, y, z, argument_array):  #change tile picture
 		return "bad"
 
 	g.maps[z].field[y][x].pix = g.tiles[argument_array[0][0][1:-1]]
+	g.maps[z].field[y][x].name = argument_array[0][0][1:-1]
 	g.maps[z].field[y][x].addpix = []
 	return 1
 
@@ -1559,6 +1242,12 @@ def script_skill(x, y, z, argument_array):  #skill functions
 		else:
 			g.player.skill[temp][5] = 1
 			return 1
+	elif argument_array[0][0][1:-1].lower() == "use":
+		if g.cur_window != "battle":
+			print "Cannot use skills outside of battle."
+			return "bad"
+		battle.useskill(temp, 1)
+		return 1
 	else:
 		print "Unknown switch " + argument_array[0][0][1:-1] + "found."
 		return "bad"
@@ -1571,10 +1260,14 @@ def script_stat(x, y, z, argument_array):  #return stat
 	if switch2 == "name": return "\"" + player.name + "\""
 	elif switch2 == "hp": return player.hp
 	elif switch2 == "ep": return player.ep
-	elif switch2 == "maxhp": return player.adj_maxhp
-	elif switch2 == "maxep": return player.adj_maxep
-	elif switch2 == "attack": return player.adj_attack
-	elif switch2 == "defense": return player.adj_defense
+	elif switch2 == "maxhp": return player.maxhp
+	elif switch2 == "maxep": return player.maxep
+	elif switch2 == "attack": return player.attack
+	elif switch2 == "defense": return player.defense
+	elif switch2 == "adj_maxhp": return player.adj_maxhp
+	elif switch2 == "adj_maxep": return player.adj_maxep
+	elif switch2 == "adj_attack": return player.adj_attack
+	elif switch2 == "adj_defense": return player.adj_defense
 	elif switch2 == "gold": return player.gold
 	elif switch2 == "exp": return player.exp
 	elif switch2 == "level": return player.level
