@@ -19,7 +19,7 @@
 #This file contains everything that needs to be accessed by most or all files.
 
 
-#Dragon Hunt 3.49. Released under the GPL. Copyright 2005
+#Dragon Hunt 3.51. Released under the GPL. Copyright 2005
 #(Note to self: when changing version, change variables.txt, README.txt,
 # and setup.py.)
 
@@ -45,6 +45,10 @@ import action
 #This will be displayed on the Title screen and Main screen.
 game_name = ""
 
+#This is the main window for the entire game. (Will be filled in with
+#a Tkinter window reference later.)
+window_main = ""
+
 #fill colour
 fill_colour = "#b9e5ad"
 fill_sel_colour = "#7aa96e"
@@ -72,15 +76,15 @@ var_list = {}
 #of bombs from inv. UNUSED
 cur_mon_hp = 0
 
-#Current window in use (main, battle, inventory)
+#Current window in use (main, battle, inventory, shop)
 cur_window = "main"
 
 #Should the player be allowed to move? Used with the move scripting command,
 #to prevent moving again after moving.
 allow_move = 1
 
-#Should the player be allowed to move? Used with the move scripting command,
-#to prevent moving again after moving.
+#Should the automatic changing of the hero picture be allowed?
+#Used with the manual hero() command, to prevent the change from being undone.
 allow_change_hero = 1
 
 #width of the hp/ep bars
@@ -103,12 +107,22 @@ binding_attack="a"
 binding_save="s"
 binding_quit="q"
 binding_inv="i"
-binding_inv_right="<Next>"
-binding_inv_left="<Prior>"
 binding_console="~"
 message_lines = 6
 difficulty = 1
 
+name_name = "Name"
+hp_name = "HP"
+ep_name = "EP"
+attack_name = "Attack"
+defense_name = "Defense"
+gold_name = "Gold"
+skill_name = "Skill"
+exp_name = "EXP"
+level_name = "Level"
+
+
+
 #Used in new_game
 default_player_name = "Alfred"
 
@@ -259,6 +273,7 @@ def init_data():
 	read_shops()
 	read_perturn()
 	load_buttons()
+        load_icons()
 
 #Get the name of the module from the variables.txt file. Note that this
 #does not set the game name, only returns it. This lets it be used in rpg.py
@@ -290,8 +305,6 @@ def read_settings():
 	global binding_save
 	global binding_quit
 	global binding_inv
-	global binding_inv_right
-	global binding_inv_left
 	global binding_console
 	global message_lines
 	global difficulty
@@ -325,10 +338,6 @@ def read_settings():
 			binding_quit = line.split("=", 1)[1].strip()
 		if (line.split("=", 1)[0].strip() == "inv"):
 			binding_inv = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "inv_right"):
-			binding_inv_right = line.split("=", 1)[1].strip()
-		if (line.split("=", 1)[0].strip() == "inv_left"):
-			binding_inv_left = line.split("=", 1)[1].strip()
 		if (line.split("=", 1)[0].strip() == "load_console"):
 			binding_console = line.split("=", 1)[1].strip()
 		if (line.split("=", 1)[0].strip() == "message_lines"):
@@ -348,22 +357,50 @@ def read_variables():
 		if (line[:1] == "#" or line[:1] == ""):
 			line = file.readline()
 			continue
-		if (line.split("=", 1)[0].strip() == "hp"):
+		switch2 = line.split("=", 1)[0].strip().lower()
+		if (switch2 == "hp"):
 			read_dice(0, line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "ep"):
+		elif (switch2 == "ep"):
 			read_dice(1, line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "attack"):
+		elif (switch2 == "attack"):
 			read_dice(2, line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "defense"):
+		elif (switch2 == "defense"):
 			read_dice(3, line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "gold"):
+		elif (switch2 == "gold"):
 			read_dice(4, line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "tilesize"):
+		elif (switch2 == "tilesize"):
 			global tilesize
 			tilesize = int(line.split("=", 1)[1].strip())
-		elif (line.split("=", 1)[0].strip() == "default_player_name"):
+		elif (switch2 == "default_player_name"):
 			global default_player_name
 			default_player_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "name_name"):
+			global name_name
+			name_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "hp_name"):
+			global hp_name
+			hp_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "ep_name"):
+			global ep_name
+			ep_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "attack_name"):
+			global attack_name
+			attack_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "defense_name"):
+			global defense_name
+			defense_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "gold_name"):
+			global gold_name
+			gold_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "skill_name"):
+			global skill_name
+			skill_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "exp_name"):
+			global exp_name
+			exp_name = line.split("=", 1)[1].strip()
+		elif (switch2 == "level_name"):
+			global level_name
+			level_name = line.split("=", 1)[1].strip()
 		line = file.readline()
 
 #given a dice set and string of the form 2d4+5, place in new_game_dice
@@ -404,15 +441,18 @@ def read_perturn():
 
 
 #skills array. Each skill is a separate line in the array. Each line goes:
-#name, effect, level, price, description, acquired, scripting.
+#name, effect, level, price, description, acquired, scripting, picture.
 #effect is an integer that tells battle.py which case in a select to pick.
 #(0=Rage, 1=Sneak, 2=Frenzy, 3=Dismember, 4=Scripted.)
 #level is the skillpoints required to get the skill,
 #price is the ep needed to use.
 #acquired tells if the skill has already been learned by the player.
+#scripting is an array (that may be empty) that describes the scripting run on use.
+#picture is the picture used to show that skill.
 
 #Add a skill to the skill[] array
-def addskill(name, effect, level, price, description, scripting=[]):
+def addskill(name, effect, level, price, description, scripting=[],
+					picture="items/rage.png"):
 	player.skill.append([])
 	i = len(player.skill)
 	player.skill[i-1].append(name)
@@ -422,6 +462,7 @@ def addskill(name, effect, level, price, description, scripting=[]):
 	player.skill[i-1].append(description)
 	player.skill[i-1].append(0)
 	player.skill[i-1].append(scripting)
+	player.skill[i-1].append(picture)
 
 
 #takes a skill name, and returns its location in the
@@ -440,20 +481,20 @@ def add_skill(skill_loc):
 		player.skill[skill_loc][5] = 1
 		return 1
 
-#Add built-in skills.
-addskill("Rage",       0, 1, 10,
-	"Gives you increased damage for the rest of a battle")
-addskill("Sneak Away", 1, 1, 10,
-	"Attempts to leave a battle.")
-addskill("Dismember",  3, 2, 20,
-	"Your next attack will do maximum damage, and ignore armor")
-addskill("Frenzy",  2, 2, 30,
-	"Your next attack will try to hit more than once")
-
 #Load the skills. Requires g.mod_directory to be set
 def read_skills():
+	#Add built-in skills.
+	addskill("Rage", 0, 1, 10, "Gives you increased damage for the"+
+		" rest of a battle", picture="items/rage.png")
+	addskill("Sneak Away", 1, 1, 10,
+		"Attempts to leave a battle.", picture="items/sneak_away.png")
+	addskill("Dismember",  3, 2, 20, "Your next attack will do maximum"+
+		" damage, and ignore armor", picture="items/bastard_sword.png")
+	addskill("Frenzy",  2, 2, 30, "Your next attack will try to hit"+
+		" more than once", picture = "items/frenzy.png")
 	if path.exists(g.mod_directory + "/data/skills.txt"):
 		temp_skills = read_script_file("/data/skills.txt")
+
 	#temp storage for the skill data
 	temp_skill_name = ""
 	temp_skill_level = 0
@@ -475,14 +516,15 @@ def read_skills():
 				#If we have any data, add the previous skill
 				if temp_skill_name != "":
 					addskill(temp_skill_name, 4, temp_skill_level,
-							temp_skill_price,
-							temp_skill_description, temp_skill_scripting)
+							temp_skill_price, temp_skill_description,
+							temp_skill_scripting, temp_skill_picture)
 
 				temp_skill_name = line_strip[1:]
 				temp_skill_level = 0
 				temp_skill_price = 0
 				temp_skill_description = ""
 				temp_skill_scripting = []
+				temp_skill_picture = "items/rage.png"
 		else:
 			if data_or_scripting == 0:
 				command = line_strip.split("=", 1)[0].lower().strip()
@@ -490,12 +532,13 @@ def read_skills():
 				if command == "level": temp_skill_level = value
 				elif command == "price": temp_skill_price = value
 				elif command == "description": temp_skill_description = value
+				elif command == "picture": temp_skill_picture = value
 			else:
 				temp_skill_scripting.append(line_strip)
 	if temp_skill_name != "":
 		addskill(temp_skill_name, 4, temp_skill_level,
-				temp_skill_price,
-				temp_skill_description, temp_skill_scripting)
+				temp_skill_price, temp_skill_description,
+				temp_skill_scripting, temp_skill_picture)
 
 
 #Rolls dice in the form 2d6, where 2 is the number of dice, and 6 the number of
@@ -613,6 +656,17 @@ def load_buttons():
 	for image_name, image in temp_images.iteritems():
 		buttons[image_name] = image
 
+icons = {}
+
+#This loads the icons
+def load_icons():
+	global icons
+	temp_images = read_images("/images/icons/")
+	icons = {}
+	for image_name, image in temp_images.iteritems():
+		icons[image_name] = image
+
+
 
 #given a filename, return the script contained in the file. from_editor will
 #be used for the map editor, to keep it from shredding formatting.
